#ifndef _TRIE_H
#define _TRIE_H

#include <stdio.h>

#ifdef DEBUG
  #define Debug fprintf
#else
  #define Debug nullprintf
#endif

#define MaxKeyLen 100
#define MAXLEVEL 8
#define INS_INC(x)

extern int line_size;
enum Boolean {False, True};
enum Identity {array, btree, hashtable};

void nullprintf(FILE * fp, char *string, ...);

typedef int keyType;
typedef int dataType;

class TrieNode;

class elementType {
 public:
  keyType key;
  dataType data;
  TrieNode* next;

  elementType() {
    data=-1;
    next=NULL;
  }

  elementType(keyType k) {
    key=k;
    data=-1;
    next=NULL;
  }
};

class Trie {
  int maxarraysize;	/* threshold for changing array to Btree */
  int btreeary;		/* width of a Btree node */
  int btreemaxheight;	/* threshold for changing Btree to hash table */
  int bucketsize;       /* hash table bucket size */
  int numbucket;        /* number of buckets */
  TrieNode* root;

  TrieNode* tnList[MaxKeyLen];	// used for traversal
  elementType* elemptr[MaxKeyLen];
  int depth;

 public:
  Trie(int arraymaxsizeIn, int btreearyIn, int btreemaxheightIn, int bucksizeIn, int numbuckIn) {
    maxarraysize=arraymaxsizeIn;
    btreeary=btreearyIn;
    btreemaxheight=btreemaxheightIn;
    bucketsize=bucksizeIn;
    numbucket=numbuckIn;
    root=NULL;
  }

  int Arraymaxsize() { return maxarraysize; }
  int Btreemaxheight() { return btreemaxheight; }
  int Bucketsize() { return bucketsize; }
  int Numbucket() { return numbucket; }

  Boolean Insert(int keyLen, keyType *keyList, dataType d);
  Boolean Delete(int keyLen, keyType *keyList);
  dataType* Search(int, keyType*);

  void print_klist(FILE*);
  void Dump(FILE*);
};

class TrieNode {
 protected:
  int size;

 public:
  virtual Identity getId()=0;
  virtual elementType* Insert(keyType)=0;
  virtual Boolean Delete(keyType)=0;
  virtual elementType* Search(keyType)=0;
  virtual elementType* getFirst()=0;
  virtual elementType* getNext()=0;
};

#endif
